----------------------------------------------
-- KeplerParamBox.lua
-- Author - Vincent Giangiulio a.k.a Vincent
-- Edit by Lygin Gennady a.k.a Croc 16.05.2014
-- Integration by Lygin Gennady a.k.a Croc 16.05.2014
-- Edit by Patrick Ziegler a.k.a. Gironde 07-04-2017
-- Edit by Patrick Ziegler a.k.a. Gironde 09-10-2018
----------------------------------------------
--require "locale"
require "KeplerParamCompareFrame"
----------------------------------------------
-- Define local functions
----------------------------------------------
local uly_to_km = 9460730.4725808


local formatNum = function(num, ndigits, units)
	if type(num) ~= "number" or num == INF then return "--" end
	if num < 0 then sign = -1 else sign = 1 end
	local lnum = 1e4
	local pow = 0
	local absnum = math.abs(num)
	if absnum < 1e-4 then
		num_str = string.format("%0."..ndigits.."f %s", absnum, units)
	elseif absnum <= 1 / lnum then
		while absnum < 1 do
			absnum = absnum * 10
			pow = pow - 1
		end
		num_str = string.format("%0."..ndigits.."f e%0.0f %s", sign * absnum, pow, units)
	elseif absnum <= 1 then
		num_str = string.format("%0.2g %s", num, units)
	elseif absnum >= lnum then
		while absnum >= 10 do
			absnum = absnum / 10
			pow = pow + 1
		end
		num_str = string.format("%0."..ndigits.."f e%0.0f %s", sign * absnum, pow, units)
	else
		num_str = string.format("%0."..ndigits.."f %s", num, units)
	end
	return num_str
end

local formatKm = function(km, ndigits)
	if not km then return "--" end
	local sign, value, units
	if km < 0 then sign = -1 else sign = 1 end
	km = math.abs(km)
	if km > 1e12 then
		value = km / KM_PER_LY
		units = _("ly")
	elseif km >= 1e7 then
		value = km / KM_PER_AU
		units = _("AU")
	else
		value = km
		units = _("km")
	end
	str = formatNum(sign * value, ndigits, units)
	return str
end

local formatKg = function(kg, ndigits)
	if not kg then return "--" end
	local sign, value, units
	local mstr = ""
	if kg < 0 then sign = -1 else sign = 1 end
	kg = math.abs(kg)
	if kg > 0.1 * MASS_OF_SOL then
		value = kg / MASS_OF_SOL
		units = _("MSun")
		mstr = formatNum(sign * value, ndigits, units)
	elseif kg > 0.9 * MASS_OF_JUPITER then
		value = kg / MASS_OF_JUPITER
		units = _("MJupiter")
		mstr = formatNum(sign * value, ndigits, units)
	elseif kg >= 0.1 * MASS_OF_EARTH then
		value = kg / MASS_OF_EARTH
		units = _("MEarth")
		mstr = formatNum(sign * value, ndigits, units)
	end
	return mstr
end

local formatDay = function(day, ndigits)
	if not day then return "--" end
	local sign, value, units
	if day < 0 then sign = -1 else sign = 1 end
	day = math.abs(day)
	if day > 365.25 then
		value = day / 365.25
		units = _("years")
		if value < 2 then
			units = _("year")
		end
	elseif day <= 1 then
		value = day * 24
		units = _("h")
	else
		value = day
		units = _("days")
	end
	str = formatNum(sign * value, ndigits, units)
	return str
end

local getSemiMajorAxis = function(r, v, T)
	local u = (v * T) / (2*PI*r)
	local F = (1 + 54 * u^2 + 6 * u * math.sqrt(3 + 81 * u^2)) ^ (1 / 3)
	local a = (r / 6) * (1 + F + (1 / F))
	return a
end

local getMass = function(T, a)
	local M = (4 * PI^2 * a^3) / (G * T^2)
	return M
end

local getExcentricity = function(T, a, r, v, vrad)
	local l = r * math.sqrt(v^2 - vrad^2)
	local e = math.sqrt(1 - ((l * T) / (2 * PI * a^2))^2)
	return e
end

local getTemperature = function(obj)
	local radtemp
	local parent = obj
	while parent:type() ~= "star" do
		parent = parent:getinfo().parent
	end
	if parent:type() == "star" then
		local objpos = obj:getposition()
		local parentpos = parent:getposition()
		local distStar = objpos:distanceto(parentpos)
		local Rstar = parent:radius()
		local Tstar = parent:getinfo().temperature -- °Kelvin
		-- local Tsol = 5780 -- Celestia EPV3 5229
		--local Tsol = 5860 -- Celestia 1.6.1
		--local UAtoKm = 149597870.700
		local Obj_albedo = obj:getinfo().albedo -- geometric albedo
		--radtemp = Tstar * math.sqrt(Rstar / (2 * distStar)) - 273.15 -- original formula Vincent Giangiulio a.k.a Vincent
		--radtemp = (Tstar/Tsol)*280*math.pow((1-Obj_albedo)/math.pow(distStar/UAtoKm,2),0.25) -- http://planet-terre.ens-lyon.fr/article/temperature-de-surface.xml
		--radtemp = Tstar* (math.pow(1-Obj_albedo,0.25) * math.sqrt(Rstar/2*distStar)) --  'onetwothree' https://celestia.space/forum/viewtopic.php?f=2&t=18332&p=140566#p140566
		local a0=Rstar / (2*distStar)
		local a2=math.sqrt(a0)
		local a1=math.pow(1-Obj_albedo,0.25) -- ok
		radtemp = Tstar*a1*a2 --  'onetwothree' https://celestia.space/forum/viewtopic.php?f=2&t=18332&p=140566#p140566
		radtemp = radtemp - 273.15
	end
	return radtemp
end

-- Speed functions
-- from Toti's OrbitalSpeeds.celx script
local tspeed = function(p0, p1, dt)
	-- tangential speed in km/s
	return (p1 - p0):length() * KM_PER_MLY / dt
end

local aspeed = function(p0, p1, dt)
	-- angular speed in rad/s
	return math.acos( (p0 * p1) / ( p0:length() * p1:length() ) ) / dt
end

local rspeed = function(p0, p1, dt)
	-- radial speed in km/s
	return (p1:length() - p0:length()) * KM_PER_MLY  / dt
end

local getKeplerParamCandidate = function(obj)
	local candidate = nil
	local objtype = obj:type()
	local objname = obj:name()
	if not empty(obj) and (objtype == "asteroid" or objtype == "comet" or objtype == "spacecraft" or objtype == "location" or
	objtype == "planet" or objtype == "dwarfplanet" or objtype == "moon" or objtype == "minormoon") then
		for k, exceptname in pairs(except2_name_t) do
			if objname == exceptname then
				return nil
			end
		end
		if objtype == "location" then
			return obj:getinfo().parent
		end
		local startdate = obj:getinfo().lifespanStart
		local enddate = obj:getinfo().lifespanEnd
		local t = celestia:gettime()
		if t > startdate and t < enddate then
			local objpos = obj:getposition()
			local parent = obj:getinfo().parent
			local parenttype = parent:type()
			local parentradius = parent:radius()
			local parentpos = parent:getposition()
			local dsurf = objpos:distanceto(parentpos) - parentradius
			if objtype == "spacecraft" and dsurf < 10 and parenttype ~= "spacecraft" then
				candidate = parent
			else
				candidate = obj
			end
		end
	end
	if not candidate then
		for k, bname in pairs(barycenter_name_t) do
			if objname == bname then
				candidate = obj
			end
		end
	end
	return candidate
end

local getObjectTypeName = function(objtype, objradius)
	local typename = ""
	if objtype == "asteroid" then
		typename = _("Asteroid")
	elseif objtype == "comet" then
		typename = _("Comet")
	elseif objtype == "spacecraft" then
		typename = _("Spacecraft")
	elseif objtype == "planet" then
		if objradius < 15000 then
			typename = _("Telluric Planet")
		else
			typename = _("Gas Giant")
		end
	elseif objtype == "dwarfplanet" then
		typename = _("Dwarf Planet")
	elseif objtype == "moon" then
		typename = _("Moon")
	elseif objtype == "minormoon" then
		typename = _("Minor Moon")
	elseif objtype == "invisible" then
		typename = _("Barycenter")
	end
	return typename
end

----------------------------------------------line221
-- Set initial values
----------------------------------------------
local newwidth, newheight
--local KeplerParamFrameWidth, KeplerParamFrameHeight = 230, 320+120
local KeplerParamFrameWidth, KeplerParamFrameHeight = 230, 518

--local KPFrameBGColor = csetbofill
--local KPFrameTitleColor = ctext
local KPFrameTextColor = cbutextoff
local KPFTextValue = cbutextoff
--local cKeplerParam = ctext -- moved in menuLine5 (global variable) by Gironde 10-04-2017
--local cKeplerParam = {0,1,0,1} -- Croc -- moved in menuLine5 (global variable) by Gironde 10-04-2017
local Kepler_param_first_tick = true
local display_Kepler_param = false

local dt_s = 1 -- time interval (seconds) used to compute velocities
local dt_d = dt_s / 86400

local earth = celestia:find("Sol/Earth")
local earthradius = earth:radius()

local Kepler_Param_obj =""
local Kepler_Param_objname = ""

-- List of valid objects for which orbital parameters won't be displayed:
except1_name_t = { "Patriarche 6" }

-- List of valid objects for which no parameter will be displayed (window disabled):
except2_name_t =  { "Freedom 7", "Liberty Bell 7" }

-- List of barycenters for which orbital parameters will be displayed:
barycenter_name_t = { "Pluto-Charon" }

----------------------------------------------
-- Set up and Draw the boxes
----------------------------------------------line255


-- command's button ---------------------------------------------
KeplerParamBox = CXBox:new()
	:init(0, 0, 0, 0)
	:movable(false)
	:attach(screenBox, width, height, 0, 0)

KeplerParamCheck.Customdraw =
	function(this)
		-- Set the initial state of the check-box from 'config.lua'.
		if Kepler_param_first_tick and enable_Kepler_param then
			KeplerParamCheck.Action()
			display_Kepler_param = true
		end
		Kepler_param_first_tick = false

		if selection then
			if newselection then
				local sel = celestia:getselection()
				Kepler_param_obj = getKeplerParamCandidate(sel)
				if not Kepler_param_obj then
					disableCheckBox(KeplerParamCheck)
					cKeplerParam = {1,1,0,1} -- yellow
					if KeplerParamFrame.Visible then
						KeplerParamFrame.Visible = false
					end
				else
					enableCheckBox(KeplerParamCheck)
					cKeplerParam = {0,1,0,1} -- green
					if display_Kepler_param and not KeplerParamFrame.Visible then
						KeplerParamFrame.Visible = true
					end
				end
			end
		else
			disableCheckBox(KeplerParamCheck)
			cKeplerParam = cbubordmenuoff -- blue
			if KeplerParamFrame.Visible then
				KeplerParamFrame.Visible = false
			end
		end
	end

-- =====================

-- frame ---------------------------------------------------------------------------
KeplerParamFrame = CXBox:new()
	:init(0, 0, 0, 0)
	-- :bordercolor(cbubordoff)
	:fillcolor(csetbofill)
	--:fillimage(fill_img_Frame2)
	:movable(true)
	:clickable(false)
	:visible(false)
	--:attach(screenBox, 2, 280, width - KeplerParamFrameWidth - 2, height - KeplerParamFrameHeight - 280-120)
	-- ybas=280 yhaut= 817-518-280 = 299-280 = 19
	-- 280+518+ (height - KeplerParamFrameHeight - 280) = 280+518+19 = 817
	:attach(screenBox, 2, 280+30, width - KeplerParamFrameWidth - 2, height - KeplerParamFrameHeight - 280-30)

closeKeplerParamButton = CXBox:new()
		:init(0, 0, 0, 0)
		--:bordercolor(cbubordoff)
		:textfont(smallfont)
		:textcolor({1,0,0,0.5})
		:textpos("center")
		:movetext(0, 12)
		:text("x")
		:movable(false)
		:active(true)
		:attach(KeplerParamFrame, 220, 290, 2, 2)

closeKeplerParamButton.Action = (function()
		return
			function()
			 display_KeplerParam = not display_KeplerParam
				if KeplerParamFrame.Visible then
					KeplerParamFrame.Visible = false
					KeplerParamCheck.Text = ""
				end
			end
end) ()


CompareButton = CXBox:new()
		:init(0, 0, 0, 0)
		:fillcolor(cFrameFill)
		--:fillimage (fill_img_Frame)
		:bordercolor(color0)
		:textfont(smallfont)
		:textcolor(cbutextoff)
		:textpos("center")
		:movetext(0, 12)
		:text("compare")
		:movable(false)
		:active(true)
		:attach(KeplerParamFrame, 5, 5, 180, 500)
		
CompareButton.Action = (function()
	return
		function()
			--display_KeplerParamCompare = not display_KeplerParamCompare
			if KeplerParamCompareFrame.Visible then
				CompareObj = nil
				Compare_Kepler_param_obj = nil
				KeplerParamCompareFrame.Visible = false
			else
				CompareObj = celestia:getselection()
				Compare_Kepler_param_obj = Kepler_param_obj
				KeplerParamCompareFrame.Visible = true
			end
		end
end) ()

KeplerParamFrame.Customdraw =
	function(this)
		if newwidth ~= width or newheight ~= height then
			newwidth, newheight = width, height
			--KeplerParamFrame:attach(screenBox, 5, height - KeplerParamFrameHeight - 150-120, width - KeplerParamFrameWidth - 5, 150)
			-- ybas=817-518-150 = 299-150 = 149 yhaut= 150
			-- 149+518+150=817
			KeplerParamFrame:attach(screenBox, 5, height - KeplerParamFrameHeight - 150-30, width - KeplerParamFrameWidth - 5, 150+30)
		end
		
		local sel = celestia:getselection()
		
		if not empty(sel) then
			Kepler_Param_objname = Kepler_param_obj:name()
			local objpos = Kepler_param_obj:getposition()
			local objradius = Kepler_param_obj:radius()
			local T_d = Kepler_param_obj:getinfo().orbitPeriod
			local T_s = T_d * 86400
			local parent
			if Kepler_Param_objname == "Pluto" or Kepler_Param_objname == "Charon"
			  or Kepler_Param_objname == "Nix" or Kepler_Param_objname == "Hydra" then
				parent = celestia:find("Sol/Pluto-Charon")
			elseif Kepler_Param_objname == "Orcus" or Kepler_Param_objname == "Vanth" then
				parent = celestia:find("Sol/Orcus-Vanth")
			else
				parent = Kepler_param_obj:getinfo().parent
			end
			local rotation_actualobj = Kepler_param_obj:getinfo().rotationPeriod

			local parentradius = parent:radius()
			local parentpos = parent:getposition()
			local parentname = parent:localname() -- parent:localname() is the name in local language
			local r = objpos:distanceto(parentpos)
			local surfdist = r - parentradius - objradius

			local earthpos = earth:getposition()
			local earthdist = 0.5 * (objpos:distanceto(earthpos) - earthradius - objradius
								+ math.abs(objpos:distanceto(earthpos) - earthradius - objradius))

			local r1 = parentpos - objpos
			local t1 = celestia:gettime()
			local t0 = t1 - dt_d
			local r0  = parent:getposition(t0) - Kepler_param_obj:getposition(t0)
			local v = tspeed (r0, r1, dt_s)
			local w = aspeed (r0, r1, dt_s)
			local vrad = rspeed(r0, r1, dt_s)

			local a = getSemiMajorAxis(r, v, T_s)
			local M = getMass(T_s, a)
			local e = getExcentricity(T_s, a, r, v, vrad)
			local aphelion = a * (1 + e)
			local perihelion = a * (1 - e)

			local radtemp = getTemperature(Kepler_param_obj)

			local startdate = Kepler_param_obj:getinfo().lifespanStart
			local UTCstart = celestia:tdbtoutc(startdate)
			local enddate = Kepler_param_obj:getinfo().lifespanEnd
			local UTCend = celestia:tdbtoutc(enddate)

			local objtype = Kepler_param_obj:type()
			if objtype == "spacecraft" then
				if T_d > 10 then
					M = nil
					a = nil
					e = nil
					aphelion = nil
					perihelion = nil
				end
				if surfdist < 0.4 then
					M = nil
					a = nil
					T_d = nil
					e = nil
					aphelion = nil
					perihelion = nil
				end
			end

			local parenttype = parent:type()
			if parenttype == "spacecraft" or parenttype == "invisible" then
				M = nil
			end

			for k, exceptname in pairs(except1_name_t) do
				if Kepler_Param_objname == exceptname then
					M = nil
					a = nil
					T_d = nil
					e = nil
					aphelion = nil
					perihelion = nil
				end
			end

-- ===========================================================================================================
-- beginning of editing
-- ===========================================================================================================
				
			--****************************************** title *******************************************
				local line = 0

				textlayout:setfont(labelfont)
				textlayout:setlinespacing(labelfont:getheight() + 2) -- +6
				textlayout:setfontcolor(ctext)
				LenghText = labelfont:getwidth("KEPLERIAN PARAMETERS")
				textlayout:setpos(this.lb+(KeplerParamFrameWidth/2)-(LenghText/2), this.tb - 14 - 11*line)
				textlayout:println("KEPLERIAN PARAMETERS")

			--****************************************** name object *************************************

			line = line + 2
				textlayout:setfont(labelfont)
				textlayout:setfontcolor({1,0,0,0.8})
				LenghText = labelfont:getwidth(Kepler_param_obj:localname())  -- (Kepler_param_obj:localname()) is the name in local language
				textlayout:setpos(this.lb+(KeplerParamFrameWidth/2)-(LenghText/2), this.tb - 14 - 11*line)
				textlayout:println(Kepler_param_obj:localname())  -- (Kepler_param_obj:localname()) is the name in local language

			--****************************************** type of object **********************************

			line = line + 2
				textlayout:setlinespacing(normalfont:getheight() + 2) -- +6
				textlayout:setfont(normalfont)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:println(_("Object Type: ")..getObjectTypeName(objtype, objradius))
				if objtype == "spacecraft" then
					local startdate_str = "--"
					local enddate_str = "--"
					if math.abs(startdate) ~= INF then
						startdate_str = string.format("%0.0f", UTCstart.day).." "..monthAbbr[UTCstart.month].." "..string.format("%0.0f", UTCstart.year)
					end
					if math.abs(enddate) ~= INF then
						enddate_str = string.format("%0.0f", UTCend.day).." "..monthAbbr[UTCend.month].." "..string.format("%0.0f", UTCend.year)
					end
					textlayout:println(_("Start / End: ")..startdate_str.." / "..enddate_str)
				else
					textlayout:println("")
				end

			--****************************************** distance to Earth *********************************
			line = line + 3
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Distance to Earth: "))
					textlayout:setfontcolor({1,0,0,0.8})
					LenghText = normalfont:getwidth("Distance to Earth: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("     "..formatKm(earthdist, 2)) -- spaces for translate lengh

			--****************************************** central body *************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Central Body: "))
					textlayout:setfontcolor({1,0,0,0.8})
					LenghText = normalfont:getwidth("Central Body: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(_("  ")..parentname.."") -- spaces for translate lengh

			--****************************************** distance to star *********************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Distance to Central Body: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Distance to Central Body: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("   "..formatKm(surfdist, 2)) -- spaces for translate lengh

			--****************************************** mass of central body *****************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Central Mass: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Central Mass: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("   "..formatNum(M, 2, _("kg")))
				
					if formatKg(M, 0) ~= "" then
						line = line + 1
						LenghText = normalfont:getwidth(_("Central Mass: "))
						textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println("   "..formatKg(M, 0)) -- spaces for translate lengh
					end

			--****************************************** instantaneous velocity ***************************

			line = line + 2
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Instantaneous Velocity: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Instantaneous Velocity: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(""..formatNum(v, 2, _("km/s"))) -- spaces for translate lengh

			--****************************************** radial velocity **********************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Radial Velocity: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Radial Velocity: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..formatNum(vrad, 2, _("km/s"))) -- spaces for translate lengh

			--****************************************** aphelion *****************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Aphelion: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Aphelion: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..formatKm(aphelion, 2)) -- spaces for translate lengh

			--****************************************** perihelion *****************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Perihelion: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Perihelion: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..formatKm(perihelion, 2)) -- spaces for translate lengh

			--****************************************** semimajoraxis **************************************

			line = line + 1
				textlayout:setfontcolor(color4)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Semi Major Axis: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Semi Major Axis: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..formatKm(a, 4)) -- spaces for translate lengh

			--****************************************** orbital period *************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Orbital Period: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Orbital Period: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("   "..formatDay(T_d, 2)) -- spaces for translate lengh

			--****************************************** excentricity ****************************************

			line = line+1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Excentricity: "))
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Excentricity: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..formatNum(e, 2, "")) -- spaces for translate lengh

			--****************************************** rotation period *************************************

			line = line+1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println("Rotation period (day): ")
					LenghText = normalfont:getwidth("Rotation period (day): ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..(math.floor(rotation_actualobj*1000)/1000)) -- spaces for translate lengh

			--****************************************** radiation temperature *******************************

			line = line + 1
				TFahrenheit = radtemp*9/5+32 -- radtemp unity °C
				TKelvin = radtemp + 273 -- TKelvin = radtemp + 273.15 or (TFahrenheit+459.67)*5/9
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Radiation Temperature: "))
					LenghText = normalfont:getwidth("Radiation Temperature: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(""..formatNum(radtemp, 0, _("°C"))) -- spaces for translate lengh
					line = line + 1
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(""..formatNum(TFahrenheit, 0, _("°F")).."/"..formatNum(TKelvin, 0, _("°K"))) -- spaces for translate lengh

			--****************************************** albedo **********************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Geo.Albedo: "))
					LenghText = normalfont:getwidth("Geo.Albedo: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..math.floor((sel:getinfo().albedo)*10000)/10000) -- spaces for translate lengh

			--****************************************** oblatness *******************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("Oblateness: "))
					LenghText = normalfont:getwidth("Oblateness: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..sel:getinfo().oblateness) -- spaces for translate lengh

			--****************************************** atmosphere height ***********************************
			if (sel:getinfo().atmosphereHeight) ~= nil then
				line = line + 1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println(_("Atmosphere height: "))
						LenghText = normalfont:getwidth("Atmosphere height: ")
						textlayout:setfontcolor(KPFTextValue)
						textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println("  "..sel:getinfo().atmosphereHeight) -- spaces for translate lengh

			--****************************************** cloud height ****************************************

				line = line + 1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println(_("Cloud height: "))
						LenghText = normalfont:getwidth("Cloud height: ")
						textlayout:setfontcolor(KPFTextValue)
						textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println("  "..sel:getinfo().atmosphereCloudHeight ) -- spaces for translate lengh

			--****************************************** cloud speed ****************************************

				line = line + 1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println(_("Cloud speed: "))
						LenghText = normalfont:getwidth("Cloud speed: ")
						textlayout:setfontcolor(KPFTextValue)
						textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println("  "..sel:getinfo().atmosphereCloudSpeed ) -- spaces for translate lengh
			else
				line = line + 3
			end

			--****************************************** rings ****************************************

			line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("rings: "))
					LenghText = normalfont:getwidth("rings: ")
					local Obj_Rings = ""
					if sel:getinfo().hasRings == true then
						Obj_Rings = "yes"
					else
						Obj_Rings = "no"
					end
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println("  "..Obj_Rings ) -- spaces for translate lengh

					--****************************************** mass *****************************************
					-- sel:getinfo().mass
					--****************************************** infoURL **************************************
					-- sel:getinfo().infoURL
					--****************************************** rotationOffset *******************************
					-- sel:getinfo().rotationOffset
					--****************************************** rotationEpoch ********************************
					-- sel:getinfo().rotationEpoch
					--****************************************** rotationObliquity ****************************
					-- sel:getinfo().rotationObliquity
					--****************************************** rotationAscendingNode ************************
					-- sel:getinfo().rotationAscendingNode
					--****************************************** rotationPrecessionRate ***********************
					-- sel:getinfo().rotationPrecessionRate

			--****************************************** time ************************************************

			local actual_tdbtime = celestia:gettime()
			local actual_UTCtime = celestia:tdbtoutc(actual_tdbtime)
			local actual_UTCJDtime = celestia:tojulianday(actual_UTCtime.year,actual_UTCtime.month,actual_UTCtime.day,actual_UTCtime.hour,actual_UTCtime.minute,actual_UTCtime.seconds)

			line = line + 2
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println(_("UTC: "))
					LenghText = normalfont:getwidth("UTC: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(actual_UTCtime.year .. "-" .. actual_UTCtime.month .. "-" .. actual_UTCtime.day .." "..actual_UTCtime.hour .. ":" .. actual_UTCtime.minute .. ":" .. math.floor(actual_UTCtime.seconds))
				line = line + 1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println(_("JD : "))
					LenghText = normalfont:getwidth("TDB: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..actual_UTCJDtime)
				line = line + 1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println(_("TDB: "))
					LenghText = normalfont:getwidth("TDB: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..actual_tdbtime)

			--****************************************** position xyz ****************************************

			--====================================================================================
			-- position x,y,z in km (distance to mass center)
			--====================================================================================

			local obs = celestia:getobserver()
			local actualobj = sel --local actualobj = celestia:getselection() -- selected object
			local selectedname = actualobj:name() -- name of selected object
				local obj_find = celestia:find(selectedname) -- object to find is selected object
				local table_actualobj = actualobj:getinfo() -- recovery of the characteristics of the object
					local class_actualobj = Kepler_param_obj:getinfo().type -- class of object

			--**************************************************************************************************

			line = line + 2
			if class_actualobj ~= "star" then
				local parent_actualobj = Kepler_param_obj:getinfo().parent -- object parent for planet to spacecraft
				
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println("(note : XYZ = data file xz-y)")

			--****************************************** position relative to the ssc file *******************

				line = line + 1
				textlayout:setfontcolor(KPFrameTextColor)
				textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
				textlayout:println("Position (km)  (ssc center: ")
					LenghText = normalfont:getwidth("Position (km)  (ssc center: ")
					textlayout:setfontcolor(color2)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(parent_actualobj:name()..")")
						textlayout:setfontcolor(KPFrameTextColor)
						LenghText = normalfont:getwidth("Position (km)  (ssc center: "..(parent_actualobj:localname()))
						textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println(")")

				--**********************************************************************************************

				line = line+1
				local actual_observer_tdbtime = obs:gettime() -- get Time in TDB
				local actual_phase = obj_find:getphase(actual_observer_tdbtime) -- get phase in TDB time
				
				if not actual_phase then
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:setfontcolor(KPFTextValue)
					textlayout:println("No phase ")
				else
					local actual_phase = obj_find:getphase(actual_observer_tdbtime) -- get phase in TDB time
					local obj_posf = actual_phase:getposition(actual_observer_tdbtime)
					local posX = math.floor(obj_posf.x*uly_to_km*1000)/1000
					local posY = math.floor(obj_posf.y*uly_to_km*1000)/1000
					local posZ = math.floor(obj_posf.z*uly_to_km*1000)/1000
										
					--line = line+1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println("X: ")
					LenghText = normalfont:getwidth("X: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..posX) -- x=X
					
					--*******************************************************************************************
					
					line = line+1
					
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println("Y: ")
					LenghText = normalfont:getwidth("Y: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..posY) -- y=Z
					
					--*******************************************************************************************
					
					line = line+1
					
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println("Z: ")
					LenghText = normalfont:getwidth("Z: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..posZ) -- z=-Y

			--****************************************** position relative to Sol ****************************

					------------------------------------------------------
					-- helio vector km
					------------------------------------------------------
					local obj_position = actualobj:getposition(actual_observer_tdbtime)
					local sunpos = celestia:find("Sol"):getposition()
					local heliovector = sunpos:vectorto(obj_position)
					local obj_helioposXkm = math.floor(heliovector:getx()*uly_to_km*1000)/1000
					local obj_helioposYkm = math.floor(heliovector:gety()*uly_to_km*1000)/1000
					local obj_helioposZkm = math.floor(heliovector:getz()*uly_to_km*1000)/1000

					--*******************************************************************************************

					line = line+2
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
						textlayout:println("Position (km)  (extrapol.center: ") -- "Helio position (km): "
					textlayout:setfontcolor(KPFTextValue)
					LenghText = normalfont:getwidth("Position (km)  (extrapol.center: ")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println("Sol")
					textlayout:setfontcolor(KPFrameTextColor)
					LenghText = normalfont:getwidth("Position (km)  (extrapol.center: Sol")
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
						textlayout:println(")")

					--*******************************************************************************************

					line = line+1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println("X: ")
					LenghText = normalfont:getwidth("X: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..obj_helioposXkm) -- x=X

					--*******************************************************************************************

					line = line+1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println("Y: ")
					LenghText = normalfont:getwidth("Y: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..obj_helioposYkm) -- y=Z

					--*******************************************************************************************

					line = line+1
					textlayout:setfontcolor(KPFrameTextColor)
					textlayout:setpos(this.lb+10, this.tb - 14 - 11*line)
					textlayout:println("Z: ")
					LenghText = normalfont:getwidth("Z: ")
					textlayout:setfontcolor(KPFTextValue)
					textlayout:setpos(this.lb+10+LenghText, this.tb - 14 - 11*line)
					textlayout:println(" "..obj_helioposZkm) -- z = -Y

					--*******************************************************************************************

					line = line+2
				end -- end of if not actual_phase
			end -- end  of if class_actualobj ~= "star"
		end -- end of if not empty(sel)
	end -- end of KeplerParamFrame.
